@@ -1,4 +1,3 @@
-
 as_jsonlist <- function(x, ns = character(), ...) {
   UseMethod("as_jsonlist")
 }
@@ -20,7 +19,8 @@ attributes_to_json <- function(x, ns = character(), prefix = "", out = list()){
 
 ## override xml2 method
 #' @importFrom xml2 xml_contents xml_name xml_attrs xml_type xml_text
-as_jsonlist.xml_node <- function(x, ns = character()) {
+#' @exportS3Method
+as_jsonlist.xml_node <- function(x, ns = character(), ...) {
   key <- xml_name(x)
   ## Treat <para> and <section> as literals
   if(key %in% c("para", "section")){
@@ -56,6 +56,7 @@ as_jsonlist.xml_node <- function(x, ns = character()) {
 }
 
 ## override xml2 method
+#' @exportS3Method
 as_jsonlist.xml_nodeset <- function(x, ns = character(), ...) {
   lapply_nodes(x, as_jsonlist, ns = ns, ...)
 }
